<?php

require_once( 'spyc.php' );

class Op5Config {
	const   basepath     = '/etc/op5/';
	
	private $apc_enabled = false; /* Sets to true if apc_fetch exists */
	private $apc_ttl     = 10;
	

	/**
	 * Create an instance of Op5Config.
	 *
	 * @return  object
	 */
	public static function factory()
	{
		return new self();
	}

	/**
	 * Return a static instance of Op5Config.
	 *
	 * @return  Op5Config instance of config object
	 */
	public static function instance()
	{
		static $instance;
		
		// Load the Auth instance
		if (empty($instance)) {
			$instance = self::factory();
		}

		return $instance;
	}

	public function __construct()
	{
		$this->apc_enabled = function_exists( 'apc_fetch' );
	}
	
	
	/*
	 * Public access
	 */
	
	public function getConfig( $parameter )
	{
		return $this->getConfigVar( explode('.',$parameter), self::basepath );
	}
	
	/**
 	 * @throws RuntimeException if file is unwritable
	 */
	public function setConfig( $parameter, $array )
	{
		$path = $this->getPathForNamespace( $parameter );
		if(false === $this->setConfigFile( $path, $array )) {
			throw new RuntimeException("Could not write to $path");
		}
	}
	

	/*
	 * Recursion
	 */
	
	protected function getConfigVar( $parameter, $path )
	{
		/* Parameter array is empty; fetch tree */
		if( count( $parameter ) == 0 ) {
			/* Path is a dir, list dir an build array */
			if( is_dir( $path ) ) {
				$value = array();
				foreach( scandir( $path ) as $entry ) {
					if( $entry[0] == '.' ) {
						continue;
					}
					if( substr( $entry, -4 ) == '.yml' && is_file($path . '/' . $entry ) ) {
						$value[substr($entry,0,-4)] = $this->getConfigFile( $path . '/' . $entry );
					}
					if( is_dir( $path . '/' . $entry ) ) {
						$value[$entry] = $this->getConfigVar(array($entry), $path);
					}
				}
				return $value;
			
			/* Path is an yml, fetch yml file */
			} else if( is_file( $path .'.yml' ) ) {
				return $this->getConfigFile($path.'.yml');
			}
			
		/* Parameter tree isn't empty, step into */
		}
		else {
			$head = array_shift( $parameter );
			
			/* head is a dir, step into, and recurse */
			if( is_dir($path . '/' . $head ) ) {
				return $this->getConfigVar( $parameter, $path . '/' . $head );
			}
			
			if( is_file( $path . '/' . $head . '.yml') ) {
				/* head is an yml file, just fetch the parameter without recursion and exit */
				$value = $this->getConfigFile( $path . '/' . $head . '.yml' );
				while( count( $parameter ) ) {
					$head = array_shift( $parameter );
					if( isset( $value[$head] ) ) {
						$value = $value[$head];
					} else {
						return null;
					}
				}
				return $value;
			} else {
				return null;
			}
		}
	}
	
	/*
	 * File access
	 */

	protected function getPathForNamespace( $namespace )
	{
		return self::basepath . $namespace . '.yml';
	}
	
	protected function getConfigFile( $path )
	{
		if( $this->apc_enabled ) {
			$array = apc_fetch( $this->apc_tag_for_path( $path ), $success );
			if( $success ) {
				return $array;
			}
		}
		
		$array = Spyc::YAMLLoad( $path );
		
		if( $this->apc_enabled ) {
			apc_store( $this->apc_tag_for_path( $path ), $array, $this->apc_ttl );
		}
		
		return $array;
	}
	
	/**
 	 * @return boolean
	 */
	protected function setConfigFile( $path, $array )
	{
		if( $this->apc_enabled ) {
			/* TODO: Use store instead... but I want to verify that it's stored correctly */
			apc_delete( $this->apc_tag_for_path( $path ) );
		}
		
		$yaml = Spyc::YAMLDump( $array );
		return (bool) file_put_contents( $path, $yaml );
	}
	
	protected function apc_tag_for_path( $path ) {
		/* Fix path with realpath before hashing. Fixes double slashes and relative paths */
	    $tag = 'op5_config_' . md5( realpath($path) );
	    return $tag;
	}
}

