<?php
require_once('config.php');

echo "\nChecking if an ldap server was previously configured.\n";

//Get contents from old ldapserverfile
$oldConfigFile = '/opt/op5sys/etc/ldapserver';
if (!is_file($oldConfigFile)){
	echo "\nCould not find an ldap configuration file. Aborting.\n";
	exit(0);
}
if (!$ldapserver = file_get_contents($oldConfigFile)) {
	exit("Could not read configuration file.\n");
}
if (strlen($ldapserver) < 1) {
	exit("\nConfiguration file is empty.\n");
}
$ldapserver = explode('LDAP_', $ldapserver);
$oldDriver = array();
//
//Standard tenplate for ldap
$ldap_template = array(
	'server' 			=> '',
	'port'				=> false,
	'encryption'		=> 'none',
	'bind_dn'			=> false,
	'bind_secret'		=> false,
	'base_dn'			=> '',
	'user_base_dn'		=> '',
	'user_filter'		=> '(|(objectClass=Account)(objectClass=posixAccount)(objectClass=user)(objectClass=inetOrgPerson))',
	'group_base_dn'		=> '',
	'group_filter'		=> '(|(objectClass=posixGroup)(objectClass=group)(objectClass=groupOfUniqueNames))',
	'groupkey'			=> 'cn',
	'group_recursive'	=> true,
	'upn_suffix'		=> '',
	'userkey'			=> 'uid',
	'userkey_is_upn'	=> false,
	'userkey_realname'	=> 'cn',
	'userkey_email'		=> 'email',
	'memberkey'			=> 'uniqueMember',
	'memberkey_is_dn'	=> '1',
	'bind_with_upn'		=> false,
	'protocol_version'	=> '3',
	'driver'			=> 'LDAP',
);

foreach ($ldapserver as $setting) {
	$pos = strpos($setting, '=');
	$key = trim(substr($setting, 0, $pos));
	$value = trim(substr($setting, $pos+1)); //Don't include = and \n
	$oldDriver[$key] = $value;
}
foreach ($oldDriver as $key => $value) {
	switch ($key) {
		case 'SERVER':
			if (strpos($value, '://') !== false) {
				$value = explode('://', $value);
				if ($value[0] == 'ldaps' || $value[0] == 'LDAPS') {
					$ldap_template['encryption'] = 'ssl';
					$value = $value[1];
				}
			}
			if (strpos($value, ':') !== false) {
				$value = explode(':', $value);
				$ldap_template['server'] = $value[0];
				$ldap_template['port'] = $value[1];
			} else {
				$ldap_template['server'] = $value;
			}
			break;
		case 'BASE':
			$ldap_template['base_dn'] = $value;
			break;
		case 'BIND_DN':
			$ldap_template['bind_dn'] = $value;
			break;
		case 'USERS':
			$ldap_template['user_base_dn'] = $value;
			break;
		case 'GROUP':
			$ldap_template['group_base_dn'] = $value;
			break;
		case 'IS_AD':
			if ($value == '1') {
				$ldap_template['bind_with_upn'] = 1;
				$ldap_template['userkey_is_upn'] = 1;
				$ldap_template['userkey'] = 'userPrincipalName';
				$ldap_template['memberkey'] = 'member';
			}
			break;
		case 'UPNSUFFIX':
			$ldap_template['upn_suffix'] = $value;
			break;
	}
}

//Get contents from bind password file
$bindpwfile = '/opt/op5sys/etc/ldapbindpw';
if (is_file($bindpwfile)) {
	if (!$bindpw = file_get_contents($bindpwfile)) {
		echo "\nCould not read bind password file!\n";
		echo "You need to set the password manually in /etc/op5/auth.yml\n";
	} else {
		$ldap_template['bind_secret'] = trim($bindpw);
	}
} else {
	echo "\nNo bind password file found!\n";
	echo "You need to set the password manually in /etc/op5/auth.yml\n"; 
}

echo "\nWriting configuration for ldap.\n\n";

//Get config from auth.yml, returns an array containing all drivers and common config
$authconf = Op5Config::instance()->getConfig('auth');
//Append the config
$authconf['ldapserver'] = $ldap_template;
//Write config
Op5Config::instance()->setConfig('auth', $authconf);
